Package: bigmemory
Version: 4.6.1
Title: Manage Massive Matrices with Shared Memory and Memory-Mapped
        Files
Authors@R: 
  c(person(given = "Michael J.",
          family = "Kane",
          role = c("aut", "cre"),
          email = "bigmemoryauthors@gmail.com",
          comment = c(ORCID = "0000-0003-1899-6662")),
    person(given = "John W.",
          family = "Emerson",
          role = "aut",
          email = "jayemerson@gmail.com"),
    person(given = "Peter",
          family = "Haverty",
          role = "aut",
          email = "haverty.peter@gene.com"),
    person(given = "Charles",
          family = "Determan",
          role = "aut",
          email = "cdetermanjr@gmail.com")
  )
Depends: R (>= 3.2.0),
Imports: bigmemory.sri, methods, utils, Rcpp, uuid (>= 1.0-2)
Enhances: biganalytics, bigtabulate
LinkingTo: BH, uuid (>= 1.0-2), Rcpp
Encoding: UTF-8
Description: Create, store, access, and manipulate massive matrices.
    Matrices are allocated to shared memory and may use memory-mapped
    files.  Packages 'biganalytics', 'bigtabulate', 'synchronicity', and
    'bigalgebra' provide advanced functionality.
License: LGPL-3 | Apache License 2.0
URL: https://github.com/kaneplusplus/bigmemory
BugReports: https://github.com/kaneplusplus/bigmemory/issues
LazyLoad: yes
Suggests: testthat, remotes
RoxygenNote: 7.1.2
NeedsCompilation: yes
Packaged: 2022-05-02 14:48:46 UTC; mike
Author: Michael J. Kane [aut, cre] (<https://orcid.org/0000-0003-1899-6662>),
  John W. Emerson [aut],
  Peter Haverty [aut],
  Charles Determan [aut]
Maintainer: Michael J. Kane <bigmemoryauthors@gmail.com>
Repository: CRAN
Date/Publication: 2022-05-02 18:02:03 UTC
Built: R 4.1.3; x86_64-w64-mingw32; 2022-05-02 23:03:32 UTC; windows
Archs: x64
