# Generated by roxygen2: do not edit by hand

export(GetMatrixSize)
export(as.big.matrix)
export(attach.big.matrix)
export(big.matrix)
export(deepcopy)
export(dir.name)
export(file.name)
export(filebacked.big.matrix)
export(flush)
export(is.big.matrix)
export(is.filebacked)
export(is.float)
export(is.nil)
export(is.readonly)
export(is.separated)
export(is.shared)
export(is.sub.big.matrix)
export(morder)
export(morderCols)
export(mpermute)
export(mpermuteCols)
export(mwhich)
export(read.big.matrix)
export(shared.name)
export(sub.big.matrix)
export(write.big.matrix)
exportClasses(big.matrix)
exportClasses(big.matrix.descriptor)
exportMethods("[")
exportMethods("[<-")
exportMethods("dimnames<-")
exportMethods(as.matrix)
exportMethods(attach.resource)
exportMethods(describe)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(head)
exportMethods(length)
exportMethods(ncol)
exportMethods(nrow)
exportMethods(print)
exportMethods(tail)
exportMethods(typeof)
import(Rcpp)
import(bigmemory.sri)
import(methods)
importFrom(stats,na.omit)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(uuid,UUIDgenerate)
useDynLib(bigmemory, .registration = TRUE)
